==> pybliographer-0.6.tar.bz2: pybliographer-0.6/Pyblio/GnomeUI/Search.py <==
# This file is part of pybliographer
# 
# Copyright (C) 1998 Frederic GOBRY
# Email : gobry@idiap.ch
# 	   
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#   
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 
# $Id: Search.py,v 1.5 1999/08/18 14:05:34 gobry Exp $

from gnome.ui import *
from gtk import *

import string, re, sys, traceback

from Pyblio import Types, Search, Config

from Pyblio.GnomeUI import Utils

import Pyblio.GnomeUI.Database
import Pyblio.TextUI


def popup_add (menu, item, action = None, argument = None):
    """ Helper to add a new menu entry """
    tmp = GtkMenuItem (item)
    if action:
        tmp.connect ("activate", action, argument)
    tmp.show ()
    menu.append (tmp)
    
    return tmp

class ItemStorage (GtkTreeItem):

    def __init__ (self, name, data):
        GtkTreeItem.__init__ (self, name)

        self.data   = data
        self.parent = None
        self.children = []
        self.tree     = None
        return

    def add (self, new):
        ret = None

        # eventually create the new tree
        if not self.tree:
            self.tree = GtkTree ()
            self.set_subtree (self.tree)
            ret = self.tree
            
        self.children.append (new)
        self.tree.append (new)

        new.parent = self
        return ret

    def remove (self, child):
        # eventually remove the item
        if self.tree:
            # recursively proceed
            child.remove_subtree ()
            
            self.tree.remove_item (child)
            self.children.remove (child)
        
            # clear an empty group
            if len (self.children) == 0:
                self.remove_subtree ()
        return

    def remove_subtree (self):
        if not self.tree: return

        # remove the children also
        for child in self.children:
            # recursively proceed
            child.remove_subtree ()
            
            self.tree.remove_item (child)
            self.children.remove (child)

        self.tree = None
        #GtkTreeItem.remove_subtree (self)
        return
    
    def search (self, name):
        if self == name:
            return self

        for other in self.children:
            ret = other.search (name)
            if ret: return ret
            
        return None
    
class SearchDialog (GtkDialog):
    """ Search Dialog """
    
    def __init__ (self, data):

        GtkDialog.__init__ (self)
        
        self.set_title ('Search')
        self.set_usize (400, 500)        
        self.connect("delete_event", self.close)

        self.data  = data
        self.pairs   = []
        
        # Buttons
        apply_b = GnomeStockButton (STOCK_PIXMAP_SEARCH)
        apply_b.connect ("clicked", self.apply)

        close_b = GnomeStockButton (STOCK_BUTTON_CLOSE)
        close_b.connect ("clicked", self.close)
        
        self.action_area.add (apply_b)
        self.action_area.add (close_b)

        # user levels
        self.notebook = GtkNotebook ()
        self.vbox.pack_start (self.notebook, expand = FALSE, fill = FALSE)

        # status
        self.status = GtkStatusbar ()
        self.status.set_border_width (5)
        
        self.vbox.pack_start (self.status, expand = FALSE, fill = FALSE)
        
        # Simple search
        table = GtkTable (2,2)
        table.set_border_width (5)
        table.set_col_spacings (5)
        
        table.attach (GtkLabel ('Field'), 0, 1, 0, 1)
        table.attach (GtkLabel ('Pattern'), 1, 2, 0, 1)
        
        self.field = GtkCombo ()
        table.attach (self.field, 0, 1, 1, 2)

        self.text = GnomeEntry ('match')
        table.attach (self.text, 1, 2, 1, 2)
        self.text.gtk_entry ().connect ("activate", self.apply)

        # fill the combo
        self.field.set_popdown_strings ([' - all - '] +
                                        list (Config.get
                                              ("base/searched").data))

        self.notebook.append_page (table, GtkLabel ('Simple Search'))

        # extended search
        hbox = GtkHBox (spacing=5)
        hbox.pack_start (GtkLabel ('Search command:'),
                         expand=FALSE, fill=FALSE)
        self.expert = GnomeEntry ('expert-search')
        self.expert.gtk_entry ().connect ("activate", self.apply)
        hbox.pack_start (self.expert)
        
        self.notebook.append_page (hbox, GtkLabel ('Expert Search'))


        # database
        self.root_tree = GtkTree ()
        self.root_tree.connect ("selection_changed", self.selection)
        
        holder = GtkScrolledWindow ()
        holder.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        holder.add_with_viewport (self.root_tree)

        self.vbox.pack_start (holder)

        self.root_item = ItemStorage ('Full database', self.data.data)
        self.root_tree.append (self.root_item)

        # connect a menu to the right button
        self.root_tree.connect ("button_press_event", self.popup_menu)

        self.menu = GtkMenu ()
        self.delete_button = popup_add (self.menu, "Delete",  self.search_delete)
        self.menu.show ()

        self.status.push (1, 'Ready to search !')
        return

    
    def apply (self, widget):
        if self.data.data is None: return
        page = self.notebook.get_current_page ()

        name = None
        
        # Expert search
        if page == 1:
            
            user_global = { 's' : Pyblio.TextUI._split_req,
                            'has' : Pyblio.TextUI.has}
            
            search = self.expert.gtk_entry ().get_text ()
            try:
                exec ('tester = ' + search, user_global)
            except:
                etype, value, tb = sys.exc_info ()
		traceback.print_exception (etype, value, tb)
                dialog = GnomeErrorDialog ('internal error during evaluation')
                dialog.run_and_close ()
                return

            test = user_global ['tester']

        # Simple Search
        elif page == 0:
            field = string.lower (self.field.entry.get_text ())
            match = self.text.gtk_entry ().get_text ()
            
            if match == '': return

            error = 0
            if field == ' - all - ' or field == '':
                test = None
                name = 'any ~ ' + match
                
                for f in Config.get ("base/searched").data:
                    try:
                        new = Search.Tester (f, match)
                    except re.error, err:
                        error = 1
                        break

                    if test:
                        test = test | new
                    else:
                        test = new
                    
            else:
                try:
                    test = Search.Tester (field, match)
                except re.error, err:
                    error = 1
                
            if error:
                dialog = GnomeErrorDialog ('while compiling %s\nerror: %s' %
                                           (match, err [0]))
                dialog.run_and_close ()
                return
            
        # No search
        else:
            return

        if name is None:
            name = str (test)
            
        self.status.pop (1)

        # get selection
        selection = self.root_tree.get_selection ()
        
        if len (selection) == 0:
            selection = self.root_item
        else:
            selection = selection [0]

        selection = self.root_item.search (selection)
        data = selection.data
        
        # actual search
        Utils.set_cursor (self, 'clock')
            
        r = data.where (test)

        Utils.set_cursor (self, 'normal')

        item = ItemStorage (name, r)
        item.show ()
        
        tree = selection.add (item)
        
        if tree:
            # if we created a new subtree
            tree.connect ("button_press_event", self.popup_menu)
            selection.expand ()

        selection.tree.select_child (item)

        self.status.pop (1)
        self.status.push (1, 'found %d matches' % len (r))

        return
    
    def close (self, * arg):
        # set the whole database
        self.root_tree.select_child (self.root_item)
        self.delete_search (self.root_item)
        self.destroy ()
        return


    def selection (self, *arg):
        selection = self.root_tree.get_selection ()

        if len (selection) == 0:
            return

        selection = selection [0]
        
        data = self.root_item.search (selection).data
        
        if data is None: return
        
        if data.key != self.data.data.key:
            Utils.set_cursor (self, 'clock')
            self.data.set (data)
            Utils.set_cursor (self, 'normal')
            
            self.status.pop (1)
            self.status.push (1, '%d matches in this node' % len (data))
            
        return

    
    def popup_menu (self, *arg):
        clist, event, = arg

        if (event.type == GDK.BUTTON_PRESS and event.button == 3):
            sel = self.root_tree.get_selection ()
            if len (sel) == 0:
                self.delete_button.set_sensitive (FALSE)
            else:
                self.delete_button.set_sensitive (TRUE)
                
            self.menu.popup (None, None, None, event.button, event.time)
        return
    

    def delete_search (self, sel):
        if sel == self.root_item:
            sel.remove_subtree ()
            return
        
        sel.parent.remove (sel)
        return
        
    def search_delete (self, *arg):
        sel = self.root_tree.get_selection () [0]
        sel = self.root_item.search (sel)
        
        self.delete_search (sel)
        return

==> pybliographer-0.9.2.tar.bz2: pybliographer-0.9.2/Pyblio/GnomeUI/Search.py <==
# This file is part of pybliographer
# 
# Copyright (C) 1998 Frederic GOBRY
# Email : gobry@idiap.ch
# 	   
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#   
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 
# $Id: Search.py,v 1.8 1999/09/15 09:29:27 gobry Exp $

from gnome.ui import *
from gtk import *

import string, re, sys, traceback, copy

from Pyblio import Types, Search, Config

from Pyblio.GnomeUI import Utils

import Pyblio.GnomeUI.Database
import Pyblio.TextUI


def popup_add (menu, item, action = None, argument = None):
    """ Helper to add a new menu entry """
    tmp = GtkMenuItem (item)
    if action:
        tmp.connect ("activate", action, argument)
    tmp.show ()
    menu.append (tmp)
    
    return tmp

class ItemStorage (GtkTreeItem):

    def __init__ (self, name, data):
        GtkTreeItem.__init__ (self, name)

        self.data   = data
        self.parent = None
        self.children = []
        self.tree     = None

        return

    def add (self, new):
        ret = None

        # eventually create the new tree
        if not self.tree:
            self.tree = GtkTree ()
            self.set_subtree (self.tree)
            ret = self.tree
            
        self.children.append (new)
        self.tree.append (new)

        new.parent = self
        return ret

    def remove (self, child):
        # eventually remove the item
        if self.tree:
            # recursively proceed
            child.remove_subtree ()
            
            self.tree.remove_item (child)
            self.children.remove (child)
        
            # clear an empty group
            if len (self.children) == 0:
                self.remove_subtree ()
        return

    def remove_subtree (self):
        if not self.tree: return

        # remove the children also
        for child in copy.copy (self.children):
            # recursively proceed
            child.remove_subtree ()
            
            self.tree.remove_item (child)
            self.children.remove (child)

        self.tree = None
        #GtkTreeItem.remove_subtree (self)
        return
    
    def search (self, name):
        if self == name:
            return self

        for other in self.children:
            ret = other.search (name)
            if ret: return ret
            
        return None
    
class SearchDialog (GtkDialog):
    """ Search Dialog """
    
    def __init__ (self, data):

        GtkDialog.__init__ (self)
        
        self.set_title ('Search')
        self.set_usize (400, 500)        
        self.connect("delete_event", self.close)

        self.data  = data
        self.pairs   = []
        
        # Buttons
        apply_b = GnomeStockButton (STOCK_PIXMAP_SEARCH)
        apply_b.connect ("clicked", self.apply)

        close_b = GnomeStockButton (STOCK_BUTTON_CLOSE)
        close_b.connect ("clicked", self.close)
        
        self.action_area.add (apply_b)
        self.action_area.add (close_b)

        # user levels
        self.notebook = GtkNotebook ()
        self.vbox.pack_start (self.notebook, expand = FALSE, fill = FALSE)

        # status
        self.status = GtkStatusbar ()
        self.status.set_border_width (5)
        
        self.vbox.pack_start (self.status, expand = FALSE, fill = FALSE)
        
        # Simple search
        table = GtkTable (2,2)
        table.set_border_width (5)
        table.set_col_spacings (5)
        
        table.attach (GtkLabel ('Field'), 0, 1, 0, 1)
        table.attach (GtkLabel ('Pattern'), 1, 2, 0, 1)
        
        self.field = GtkCombo ()
        table.attach (self.field, 0, 1, 1, 2)

        self.text = GnomeEntry ('match')
        table.attach (self.text, 1, 2, 1, 2)
        self.text.gtk_entry ().connect ("activate", self.apply)

        # fill the combo
        self.field.set_popdown_strings ([' - any field - '] +
                                        list (Config.get
                                              ("base/searched").data) +
                                        [' - type - ', ' - key - '])

        self.notebook.append_page (table, GtkLabel ('Simple Search'))
        
        # extended search
        hbox = GtkHBox (spacing=5)
        hbox.pack_start (GtkLabel ('Search command:'),
                         expand=FALSE, fill=FALSE)
        self.expert = GnomeEntry ('expert-search')
        self.expert.gtk_entry ().connect ("activate", self.apply)
        hbox.pack_start (self.expert)
        
        self.notebook.append_page (hbox, GtkLabel ('Expert Search'))


        # database
        self.root_tree = GtkTree ()
        self.root_tree.connect ("selection_changed", self.selection)
        
        holder = GtkScrolledWindow ()
        holder.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        holder.add_with_viewport (self.root_tree)

        self.vbox.pack_start (holder)

        self.root_item = ItemStorage ('Full database', self.data.data)
        self.root_tree.append (self.root_item)

        # connect a menu to the right button
        self.root_tree.connect ("button_press_event", self.popup_menu)

        self.menu = GtkMenu ()
        self.delete_button = popup_add (self.menu, "Delete",  self.search_delete)
        self.menu.show ()

        self.status.push (1, 'Ready to search !')
        return

    
    def apply (self, widget):
        if self.data.data is None: return
        page = self.notebook.get_current_page ()

        name = None
        
        # Expert search
        if page == 1:
            
            user_global = {
                's'   : Pyblio.TextUI._split_req,
                'has' : Pyblio.TextUI.has,
                'has_key'  : Pyblio.TextUI.has_key,
                'has_type' : Pyblio.TextUI.has_type,
                }
            
            search = self.expert.gtk_entry ().get_text ()
            try:
                exec ('tester = ' + search, user_global)
            except:
                etype, value, tb = sys.exc_info ()
		traceback.print_exception (etype, value, tb)
                dialog = GnomeErrorDialog ('internal error during evaluation')
                dialog.run_and_close ()
                return

            test = user_global ['tester']

        # Simple Search
        elif page == 0:
            field = string.lower (self.field.entry.get_text ())
            match = self.text.gtk_entry ().get_text ()
            
            if match == '': return

            error = 0
            if field == ' - any field - ' or field == '':
                test = None
                name = 'any ~ ' + match
                
                for f in Config.get ("base/searched").data:
                    try:
                        new = Search.Tester (f, match)
                    except re.error, err:
                        error = 1
                        break

                    if test:
                        test = test | new
                    else:
                        test = new

            elif field == ' - type - ':
                # get the type description
                the_type = Types.getentry (match, 0)

                if the_type is None:
                    err = ['No such Entry type']
                    error = 1
                else:
                    try:
                        test = Search.TypeTester (the_type)
                    except re.error, err:
                        error = 1

            elif field == ' - key - ':
                try:
                    test = Search.KeyTester (match)
                except re.error, err:
                    error = 1

            else:
                try:
                    test = Search.Tester (field, match)
                except re.error, err:
                    error = 1
                
            if error:
                dialog = GnomeErrorDialog ('while compiling %s\nerror: %s' %
                                           (match, err [0]))
                dialog.run_and_close ()
                return
            
        # No search
        else:
            return

        if name is None:
            name = str (test)
            
        self.status.pop (1)

        # get selection
        selection = self.root_tree.get_selection ()
        
        if len (selection) == 0:
            selection = self.root_item
        else:
            selection = selection [0]

        selection = self.root_item.search (selection)
        data = selection.data
        
        # actual search
        Utils.set_cursor (self, 'clock')
            
        r = data.where (test)

        Utils.set_cursor (self, 'normal')

        item = ItemStorage (name, r)
        item.show ()
        
        tree = selection.add (item)
        
        if tree:
            # if we created a new subtree
            tree.connect ("button_press_event", self.popup_menu)
            selection.expand ()

        selection.tree.select_child (item)

        self.status.pop (1)
        self.status.push (1, 'found %d matches' % len (r))

        return
    
    def close (self, * arg):
        self.hide ()
        
        return 1


    def selection (self, *arg):
        selection = self.root_tree.get_selection ()

        if len (selection) == 0:
            return

        selection = selection [0]
        
        data = self.root_item.search (selection).data
        
        if data is None: return
        
        if data.key != self.data.data.key:
            Utils.set_cursor (self, 'clock')
            self.data.set (data)
            Utils.set_cursor (self, 'normal')
            
            self.status.pop (1)
            self.status.push (1, '%d matches in this node' % len (data))
            
        return

    
    def popup_menu (self, *arg):
        clist, event, = arg

        if (event.type == GDK.BUTTON_PRESS and event.button == 3):
            sel = self.root_tree.get_selection ()
            if len (sel) == 0:
                self.delete_button.set_sensitive (FALSE)
            else:
                self.delete_button.set_sensitive (TRUE)
                
            self.menu.popup (None, None, None, event.button, event.time)
        return
    

    def delete_search (self, sel):
        if sel == self.root_item:
            sel.remove_subtree ()
            return
        
        sel.parent.remove (sel)
        return
        
    def search_delete (self, *arg):
        sel = self.root_tree.get_selection () [0]
        sel = self.root_item.search (sel)
        
        self.delete_search (sel)
        return
